import { app, firestore } from "firebase-admin";
import TortoiseCollectionReference from "./TortoiseCollectionReference";
import App = app.App;
import Firestore = firestore.Firestore;

export default class TortoiseClient {
  /**
   * Firebase app
   * @readonly
   * @type {App}
   */
  readonly app: App;

  /**
   * Firestore instance
   * @readonly
   * @type {Firestore}
   */
  readonly firestore: Firestore;

  /**
   * Create a new TortoiseClient
   * @param {App} app - firebase app
   */
  constructor(app: App) {
    this.app = app;
    this.firestore = app.firestore();
  }

  /**
   * Create a new TortoiseCollectionReference
   * @template T
   * @param {string} collectionPath - collection's path
   * @return {TortoiseCollectionReference<T>}
   */
  collection<T>(collectionPath: string): TortoiseCollectionReference<T> {
    const ref = this.firestore.collection(collectionPath);
    return new TortoiseCollectionReference<T>(ref, this.firestore);
  }
}